/**
 **	File ......... RemoteSocket.cpp
 **	Published ....  2005-03-23
 **	Author ....... grymse@alhem.net
**/
/*
Copyright (C) 2004  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
//#include <stdio.h>

#include "Socks4Handler.h"
#include "Socks4Socket.h"
#include "RemoteSocket.h"




RemoteSocket::RemoteSocket(SocketHandler& h)
:TcpSocket(h)
,m_pRemote(NULL)
{
}


RemoteSocket::~RemoteSocket()
{
}


void RemoteSocket::OnConnect()
{
	if (m_pRemote && Handler().Valid(m_pRemote))
	{
		m_pRemote -> Connected();
	}
	else
	{
		Handler().LogError(this, "OnConnect", 0, "Remote not valid", LOG_LEVEL_FATAL);
		SetCloseAndDelete();
	}
}


void RemoteSocket::OnConnectFailed()
{
	if (m_pRemote && Handler().Valid(m_pRemote))
	{
		m_pRemote -> ConnectFailed();
	}
	else
	{
		Handler().LogError(this, "OnConnectFailed", 0, "Remote not valid", LOG_LEVEL_FATAL);
		SetCloseAndDelete();
	}
}


void RemoteSocket::OnRead()
{
	TcpSocket::OnRead();
	size_t l = ibuf.GetLength();
	char buf[TCP_BUFSIZE_READ];
	ibuf.Read(buf, l);
	if (m_pRemote && Handler().Valid(m_pRemote))
	{
		m_pRemote -> SendBuf(buf, l);
	}
	else
	{
		Handler().LogError(this, "OnRead", 0, "Remote not valid", LOG_LEVEL_FATAL);
		SetCloseAndDelete();
	}
}


void RemoteSocket::OnAccept()
{
	Socket *p0 = static_cast<Socks4Handler&>(Handler()).GetSocks4( GetParent() );
	Socks4Socket *pRemote = dynamic_cast<Socks4Socket *>(p0);
	if (p0 && Handler().Valid(p0) && pRemote)
	{
		SetRemote(pRemote);
		pRemote -> SetRemote(this);
		m_pRemote -> Accept();
	}
	else
	{
		Handler().LogError(this, "OnAccept", 0, "Remote not valid", LOG_LEVEL_FATAL);
		SetCloseAndDelete();
	}
}


